package java.beans;

import javax.swing.plaf.ColorUIResource;
import javax.swing.Icon;
import javax.swing.border.MatteBorder;
import javax.swing.JMenu;
import javax.swing.Box;
import javax.swing.JTabbedPane;
import javax.swing.ToolTipManager;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import java.awt.MenuBar;
import java.awt.Menu;
import java.awt.Choice;
import javax.swing.JLayeredPane;
import java.awt.BorderLayout;
import javax.swing.JScrollPane;
import java.awt.Container;
import java.awt.Color;
import java.awt.Window;
import java.awt.Component;
import java.awt.MenuShortcut;
import sun.reflect.misc.ReflectUtil;
import java.lang.reflect.Modifier;
import java.awt.AWTKeyStroke;
import java.awt.font.TextAttribute;
import java.awt.Font;
import java.awt.Insets;
import java.util.Iterator;
import java.util.EnumSet;
import java.util.EnumMap;
import java.util.TreeMap;
import java.util.SortedMap;
import java.util.HashMap;
import java.util.TreeSet;
import java.util.SortedSet;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Set;
import java.util.List;
import java.lang.reflect.InvocationTargetException;
import java.util.Date;
import java.lang.reflect.Method;
import com.sun.beans.finder.PrimitiveWrapperMap;
import java.lang.reflect.InvocationHandler;
import java.util.Vector;
import java.util.Objects;
import java.lang.reflect.Array;
import java.util.Collections;
import java.util.WeakHashMap;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.lang.reflect.Constructor;
import java.lang.reflect.Proxy;
import java.util.Hashtable;
import java.lang.reflect.Field;
import java.util.Map;

class MetaData
{
    private static final Map<String, Field> fields;
    private static Hashtable<String, PersistenceDelegate> internalPersistenceDelegates;
    private static PersistenceDelegate nullPersistenceDelegate;
    private static PersistenceDelegate enumPersistenceDelegate;
    private static PersistenceDelegate primitivePersistenceDelegate;
    private static PersistenceDelegate defaultPersistenceDelegate;
    private static PersistenceDelegate arrayPersistenceDelegate;
    private static PersistenceDelegate proxyPersistenceDelegate;
    
    MetaData() {
    }
    
    public static synchronized PersistenceDelegate getPersistenceDelegate(Class cl) {
        if (cl == null) {
            return MetaData.nullPersistenceDelegate;
        }
        if (Enum.class.isAssignableFrom(cl)) {
            return MetaData.enumPersistenceDelegate;
        }
        if (null != XMLEncoder.primitiveTypeFor(cl)) {
            return MetaData.primitivePersistenceDelegate;
        }
        if (cl.isArray()) {
            if (MetaData.arrayPersistenceDelegate == null) {
                MetaData.arrayPersistenceDelegate = new MetaData.ArrayPersistenceDelegate();
            }
            return MetaData.arrayPersistenceDelegate;
        }
        else {
            try {
                if (Proxy.isProxyClass(cl)) {
                    if (MetaData.proxyPersistenceDelegate == null) {
                        MetaData.proxyPersistenceDelegate = new MetaData.ProxyPersistenceDelegate();
                    }
                    return MetaData.proxyPersistenceDelegate;
                }
            }
            catch (Exception ex) {}
            String name = cl.getName();
            PersistenceDelegate persistenceDelegate = (PersistenceDelegate)getBeanAttribute(cl, "persistenceDelegate");
            if (persistenceDelegate == null) {
                persistenceDelegate = (PersistenceDelegate)MetaData.internalPersistenceDelegates.get(name);
                if (persistenceDelegate != null) {
                    return persistenceDelegate;
                }
                MetaData.internalPersistenceDelegates.put(name, MetaData.defaultPersistenceDelegate);
                try {
                    persistenceDelegate = (PersistenceDelegate)Class.forName("java.beans.MetaData$" + cl.getName().replace('.', '_') + "_PersistenceDelegate").newInstance();
                    MetaData.internalPersistenceDelegates.put(name, persistenceDelegate);
                }
                catch (ClassNotFoundException ex2) {
                    String[] constructorProperties = getConstructorProperties(cl);
                    if (constructorProperties != null) {
                        persistenceDelegate = new DefaultPersistenceDelegate(constructorProperties);
                        MetaData.internalPersistenceDelegates.put(name, persistenceDelegate);
                    }
                }
                catch (Exception obj) {
                    System.err.println("Internal error: " + obj);
                }
            }
            return (persistenceDelegate != null) ? persistenceDelegate : MetaData.defaultPersistenceDelegate;
        }
    }
    
    private static String[] getConstructorProperties(Class<?> clazz) {
        String[] array = null;
        int length = 0;
        for (Constructor constructor : clazz.getConstructors()) {
            String[] annotationValue = getAnnotationValue(constructor);
            if (annotationValue != null && length < annotationValue.length && isValid(constructor, annotationValue)) {
                array = annotationValue;
                length = annotationValue.length;
            }
        }
        return array;
    }
    
    private static String[] getAnnotationValue(Constructor<?> constructor) {
        ConstructorProperties constructorProperties = (ConstructorProperties)constructor.getAnnotation(ConstructorProperties.class);
        return (constructorProperties != null) ? constructorProperties.value() : null;
    }
    
    private static boolean isValid(Constructor<?> constructor, String[] array) {
        if (array.length != constructor.getParameterTypes().length) {
            return false;
        }
        int length = array.length;
        int i = 0;
        while (i < length) {
            if (array[i] == null) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    private static Object getBeanAttribute(Class<?> clazz, String s) {
        try {
            return Introspector.getBeanInfo(clazz).getBeanDescriptor().getValue(s);
        }
        catch (IntrospectionException ex) {
            return null;
        }
    }
    
    static Object getPrivateFieldValue(Object obj, String s) {
        Field field = (Field)MetaData.fields.get(s);
        if (field == null) {
            int lastIndex = s.lastIndexOf(46);
            field = (Field)AccessController.doPrivileged(new PrivilegedAction<Field>() {
                
                @Override
                public Field run() {
                    try {
                        Field declaredField = Class.forName(className).getDeclaredField(fieldName);
                        declaredField.setAccessible(true);
                        return declaredField;
                    }
                    catch (ClassNotFoundException cause) {
                        throw new IllegalStateException("Could not find class", cause);
                    }
                    catch (NoSuchFieldException cause2) {
                        throw new IllegalStateException("Could not find field", cause2);
                    }
                }
            });
            MetaData.fields.put(s, field);
        }
        try {
            return field.get(obj);
        }
        catch (IllegalAccessException cause) {
            throw new IllegalStateException("Could not get value of the field", cause);
        }
    }
    
    static {
        fields = Collections.synchronizedMap(new WeakHashMap());
        MetaData.internalPersistenceDelegates = new Hashtable();
        MetaData.nullPersistenceDelegate = new MetaData.NullPersistenceDelegate();
        MetaData.enumPersistenceDelegate = new MetaData.EnumPersistenceDelegate();
        MetaData.primitivePersistenceDelegate = new MetaData.PrimitivePersistenceDelegate();
        MetaData.defaultPersistenceDelegate = new DefaultPersistenceDelegate();
        MetaData.internalPersistenceDelegates.put("java.net.URI", new MetaData.PrimitivePersistenceDelegate());
        MetaData.internalPersistenceDelegates.put("javax.swing.plaf.BorderUIResource$MatteBorderUIResource", new MetaData.javax_swing_border_MatteBorder_PersistenceDelegate());
        MetaData.internalPersistenceDelegates.put("javax.swing.plaf.FontUIResource", new MetaData.java_awt_Font_PersistenceDelegate());
        MetaData.internalPersistenceDelegates.put("javax.swing.KeyStroke", new MetaData.java_awt_AWTKeyStroke_PersistenceDelegate());
        MetaData.internalPersistenceDelegates.put("java.sql.Date", new MetaData.java_util_Date_PersistenceDelegate());
        MetaData.internalPersistenceDelegates.put("java.sql.Time", new MetaData.java_util_Date_PersistenceDelegate());
        MetaData.internalPersistenceDelegates.put("java.util.JumboEnumSet", new MetaData.java_util_EnumSet_PersistenceDelegate());
        MetaData.internalPersistenceDelegates.put("java.util.RegularEnumSet", new MetaData.java_util_EnumSet_PersistenceDelegate());
    }
    
    static final class NullPersistenceDelegate extends PersistenceDelegate
    {
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
        }
        
        @Override
        protected Expression instantiate(Object o, Encoder encoder) {
            return null;
        }
        
        @Override
        public void writeObject(Object o, Encoder encoder) {
        }
    }
    
    static final class EnumPersistenceDelegate extends PersistenceDelegate
    {
        @Override
        protected boolean mutatesTo(Object o, Object o2) {
            return o == o2;
        }
        
        @Override
        protected Expression instantiate(Object o, Encoder encoder) {
            Enum<Object> enum1 = (Enum)o;
            return new Expression(enum1, Enum.class, "valueOf", new Object[] { enum1.getDeclaringClass(), enum1.name() });
        }
    }
    
    static final class PrimitivePersistenceDelegate extends PersistenceDelegate
    {
        @Override
        protected boolean mutatesTo(Object o, Object obj) {
            return o.equals(obj);
        }
        
        @Override
        protected Expression instantiate(Object o, Encoder encoder) {
            return new Expression(o, o.getClass(), "new", new Object[] { o.toString() });
        }
    }
    
    static final class ArrayPersistenceDelegate extends PersistenceDelegate
    {
        @Override
        protected boolean mutatesTo(Object o, Object o2) {
            return o2 != null && o.getClass() == o2.getClass() && Array.getLength(o) == Array.getLength(o2);
        }
        
        @Override
        protected Expression instantiate(Object o, Encoder encoder) {
            return new Expression(o, Array.class, "newInstance", new Object[] { o.getClass().getComponentType(), new Integer(Array.getLength(o)) });
        }
        
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
            for (int length = Array.getLength(o), i = 0; i < length; ++i) {
                Integer o3 = new Integer(i);
                Expression o4 = new Expression(o, "get", new Object[] { o3 });
                Expression o5 = new Expression(o2, "get", new Object[] { o3 });
                try {
                    Object value = o4.getValue();
                    Object value2 = o5.getValue();
                    encoder.writeExpression(o4);
                    if (!Objects.equals(value2, encoder.get(value))) {
                        DefaultPersistenceDelegate.invokeStatement(o, "set", new Object[] { o3, value }, encoder);
                    }
                }
                catch (Exception ex) {
                    encoder.getExceptionListener().exceptionThrown(ex);
                }
            }
        }
    }
    
    static final class ProxyPersistenceDelegate extends PersistenceDelegate
    {
        @Override
        protected Expression instantiate(Object o, Encoder encoder) {
            Class<Object> class1 = o.getClass();
            InvocationHandler invocationHandler = Proxy.getInvocationHandler((Object)o);
            if ((invocationHandler instanceof EventHandler)) {
                EventHandler eventHandler = (EventHandler)invocationHandler;
                Vector<String> vector = new Vector();
                vector.add(class1.getInterfaces()[0]);
                vector.add(eventHandler.getTarget());
                vector.add(eventHandler.getAction());
                if (eventHandler.getEventPropertyName() != null) {
                    vector.add(eventHandler.getEventPropertyName());
                }
                if (eventHandler.getListenerMethodName() != null) {
                    vector.setSize(4);
                    vector.add(eventHandler.getListenerMethodName());
                }
                return new Expression(o, EventHandler.class, "create", vector.toArray());
            }
            else {
                return new Expression(o, Proxy.class, "newProxyInstance", new Object[] { class1.getClassLoader(), class1.getInterfaces(), invocationHandler });
            }
        }
    }
    
    static final class java_lang_String_PersistenceDelegate extends PersistenceDelegate
    {
        @Override
        protected Expression instantiate(Object o, Encoder encoder) {
            return null;
        }
        
        @Override
        public void writeObject(Object o, Encoder encoder) {
        }
    }
    
    static final class java_lang_Class_PersistenceDelegate extends PersistenceDelegate
    {
        @Override
        protected boolean mutatesTo(Object o, Object obj) {
            return o.equals(obj);
        }
        
        @Override
        protected Expression instantiate(Object o, Encoder encoder) {
            Class<Object> obj = (Class)o;
            if (obj.isPrimitive()) {
                Field declaredField = null;
                try {
                    declaredField = PrimitiveWrapperMap.getType(obj.getName()).getDeclaredField("TYPE");
                }
                catch (NoSuchFieldException ex) {
                    System.err.println("Unknown primitive type: " + obj);
                }
                return new Expression(o, declaredField, "get", new Object[] { null });
            }
            if (o == String.class) {
                return new Expression(o, "", "getClass", new Object[0]);
            }
            if (o == Class.class) {
                return new Expression(o, String.class, "getClass", new Object[0]);
            }
            Expression expression = new Expression(o, Class.class, "forName", new Object[] { obj.getName() });
            expression.loader = obj.getClassLoader();
            return expression;
        }
    }
    
    static final class java_lang_reflect_Field_PersistenceDelegate extends PersistenceDelegate
    {
        @Override
        protected boolean mutatesTo(Object o, Object obj) {
            return o.equals(obj);
        }
        
        @Override
        protected Expression instantiate(Object o, Encoder encoder) {
            Field field = (Field)o;
            return new Expression(o, field.getDeclaringClass(), "getField", new Object[] { field.getName() });
        }
    }
    
    static final class java_lang_reflect_Method_PersistenceDelegate extends PersistenceDelegate
    {
        @Override
        protected boolean mutatesTo(Object o, Object obj) {
            return o.equals(obj);
        }
        
        @Override
        protected Expression instantiate(Object o, Encoder encoder) {
            Method method = (Method)o;
            return new Expression(o, method.getDeclaringClass(), "getMethod", new Object[] { method.getName(), method.getParameterTypes() });
        }
    }
    
    static class java_util_Date_PersistenceDelegate extends PersistenceDelegate
    {
        @Override
        protected boolean mutatesTo(Object o, Object o2) {
            return super.mutatesTo(o, o2) && ((Date)o).getTime() == ((Date)o2).getTime();
        }
        
        @Override
        protected Expression instantiate(Object o, Encoder encoder) {
            Date date = (Date)o;
            return new Expression(date, date.getClass(), "new", new Object[] { Long.valueOf(date.getTime()) });
        }
    }
    
    static final class java_sql_Timestamp_PersistenceDelegate extends MetaData.java_util_Date_PersistenceDelegate
    {
        private static final Method getNanosMethod;
        
        private static Method getNanosMethod() {
            try {
                return Class.forName("java.sql.Timestamp", true, null).getMethod("getNanos", new Class[0]);
            }
            catch (ClassNotFoundException ex) {
                return null;
            }
            catch (NoSuchMethodException detailMessage) {
                throw new AssertionError(detailMessage);
            }
        }
        
        private static int getNanos(Object obj) {
            if (MetaData.java_sql_Timestamp_PersistenceDelegate.getNanosMethod == null) {
                throw new AssertionError("Should not get here");
            }
            try {
                return (int)MetaData.java_sql_Timestamp_PersistenceDelegate.getNanosMethod.invoke(obj, new Object[0]);
            }
            catch (InvocationTargetException detailMessage) {
                Throwable cause = detailMessage.getCause();
                if ((cause instanceof RuntimeException)) {
                    throw (RuntimeException)cause;
                }
                if ((cause instanceof Error)) {
                    throw (Error)cause;
                }
                throw new AssertionError(detailMessage);
            }
            catch (IllegalAccessException detailMessage2) {
                throw new AssertionError(detailMessage2);
            }
        }
        
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
            int nanos = getNanos(o);
            if (nanos != getNanos(o2)) {
                encoder.writeStatement(new Statement(o, "setNanos", new Object[] { Integer.valueOf(nanos) }));
            }
        }
        
        static {
            getNanosMethod = getNanosMethod();
        }
    }
    
    private abstract static class java_util_Collections extends PersistenceDelegate
    {
        @Override
        protected boolean mutatesTo(Object o, Object obj) {
            if (!super.mutatesTo(o, obj)) {
                return false;
            }
            if (o instanceof List || o instanceof Set || o instanceof Map) {
                return o.equals(obj);
            }
            Collection<Object> collection = (Collection)o;
            Collection<Object> collection2 = (Collection)obj;
            return collection.size() == collection2.size() && collection.containsAll(collection2);
        }
        
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
        }
        
        static final class EmptyList_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "emptyList", null);
            }
        }
        
        static final class EmptySet_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "emptySet", null);
            }
        }
        
        static final class EmptyMap_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "emptyMap", null);
            }
        }
        
        static final class SingletonList_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "singletonList", new Object[] { ((List)o).get(0) });
            }
        }
        
        static final class SingletonSet_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "singleton", new Object[] { ((Set)o).iterator().next() });
            }
        }
        
        static final class SingletonMap_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                Map<Object, Object> map = (Map)o;
                Object next = map.keySet().iterator().next();
                return new Expression(o, Collections.class, "singletonMap", new Object[] { next, map.get(next) });
            }
        }
        
        static final class UnmodifiableCollection_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "unmodifiableCollection", new Object[] { new ArrayList((Collection)o) });
            }
        }
        
        static final class UnmodifiableList_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "unmodifiableList", new Object[] { new LinkedList((Collection)o) });
            }
        }
        
        static final class UnmodifiableRandomAccessList_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "unmodifiableList", new Object[] { new ArrayList((Collection)o) });
            }
        }
        
        static final class UnmodifiableSet_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "unmodifiableSet", new Object[] { new HashSet((Set)o) });
            }
        }
        
        static final class UnmodifiableSortedSet_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "unmodifiableSortedSet", new Object[] { new TreeSet((SortedSet)o) });
            }
        }
        
        static final class UnmodifiableMap_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "unmodifiableMap", new Object[] { new HashMap((Map)o) });
            }
        }
        
        static final class UnmodifiableSortedMap_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "unmodifiableSortedMap", new Object[] { new TreeMap((SortedMap)o) });
            }
        }
        
        static final class SynchronizedCollection_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "synchronizedCollection", new Object[] { new ArrayList((Collection)o) });
            }
        }
        
        static final class SynchronizedList_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "synchronizedList", new Object[] { new LinkedList((Collection)o) });
            }
        }
        
        static final class SynchronizedRandomAccessList_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "synchronizedList", new Object[] { new ArrayList((Collection)o) });
            }
        }
        
        static final class SynchronizedSet_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "synchronizedSet", new Object[] { new HashSet((Set)o) });
            }
        }
        
        static final class SynchronizedSortedSet_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "synchronizedSortedSet", new Object[] { new TreeSet((SortedSet)o) });
            }
        }
        
        static final class SynchronizedMap_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "synchronizedMap", new Object[] { new HashMap((Map)o) });
            }
        }
        
        static final class SynchronizedSortedMap_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "synchronizedSortedMap", new Object[] { new TreeMap((SortedMap)o) });
            }
        }
        
        static final class CheckedCollection_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "checkedCollection", new Object[] { new ArrayList((Collection)o), MetaData.getPrivateFieldValue(o, "java.util.Collections$CheckedCollection.type") });
            }
        }
        
        static final class CheckedList_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "checkedList", new Object[] { new LinkedList((Collection)o), MetaData.getPrivateFieldValue(o, "java.util.Collections$CheckedCollection.type") });
            }
        }
        
        static final class CheckedRandomAccessList_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "checkedList", new Object[] { new ArrayList((Collection)o), MetaData.getPrivateFieldValue(o, "java.util.Collections$CheckedCollection.type") });
            }
        }
        
        static final class CheckedSet_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "checkedSet", new Object[] { new HashSet((Set)o), MetaData.getPrivateFieldValue(o, "java.util.Collections$CheckedCollection.type") });
            }
        }
        
        static final class CheckedSortedSet_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "checkedSortedSet", new Object[] { new TreeSet((SortedSet)o), MetaData.getPrivateFieldValue(o, "java.util.Collections$CheckedCollection.type") });
            }
        }
        
        static final class CheckedMap_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "checkedMap", new Object[] { new HashMap((Map)o), MetaData.getPrivateFieldValue(o, "java.util.Collections$CheckedMap.keyType"), MetaData.getPrivateFieldValue(o, "java.util.Collections$CheckedMap.valueType") });
            }
        }
        
        static final class CheckedSortedMap_PersistenceDelegate extends MetaData.java_util_Collections
        {
            @Override
            protected Expression instantiate(Object o, Encoder encoder) {
                return new Expression(o, Collections.class, "checkedSortedMap", new Object[] { new TreeMap((SortedMap)o), MetaData.getPrivateFieldValue(o, "java.util.Collections$CheckedMap.keyType"), MetaData.getPrivateFieldValue(o, "java.util.Collections$CheckedMap.valueType") });
            }
        }
    }
    
    static final class java_util_EnumMap_PersistenceDelegate extends PersistenceDelegate
    {
        @Override
        protected boolean mutatesTo(Object o, Object o2) {
            return super.mutatesTo(o, o2) && getType(o) == getType(o2);
        }
        
        @Override
        protected Expression instantiate(Object o, Encoder encoder) {
            return new Expression(o, EnumMap.class, "new", new Object[] { getType(o) });
        }
        
        private static Object getType(Object o) {
            return MetaData.getPrivateFieldValue(o, "java.util.EnumMap.keyType");
        }
    }
    
    static final class java_util_EnumSet_PersistenceDelegate extends PersistenceDelegate
    {
        @Override
        protected boolean mutatesTo(Object o, Object o2) {
            return super.mutatesTo(o, o2) && getType(o) == getType(o2);
        }
        
        @Override
        protected Expression instantiate(Object o, Encoder encoder) {
            return new Expression(o, EnumSet.class, "noneOf", new Object[] { getType(o) });
        }
        
        private static Object getType(Object o) {
            return MetaData.getPrivateFieldValue(o, "java.util.EnumSet.elementType");
        }
    }
    
    static class java_util_Collection_PersistenceDelegate extends DefaultPersistenceDelegate
    {
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
            Collection<Object> collection = (Collection)o;
            if (((Collection)o2).size() != 0) {
                DefaultPersistenceDelegate.invokeStatement(o, "clear", new Object[0], encoder);
            }
            Iterator<Object> iterator = collection.iterator();
            while (iterator.hasNext()) {
                DefaultPersistenceDelegate.invokeStatement(o, "add", new Object[] { iterator.next() }, encoder);
            }
        }
    }
    
    static class java_util_List_PersistenceDelegate extends DefaultPersistenceDelegate
    {
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
            List<Object> list = (List)o;
            List<Object> list2 = (List)o2;
            int size = list.size();
            int i = (list2 == null) ? 0 : list2.size();
            if (size < i) {
                DefaultPersistenceDelegate.invokeStatement(o, "clear", new Object[0], encoder);
                i = 0;
            }
            for (int j = 0; j < i; ++j) {
                Integer o3 = new Integer(j);
                Expression o4 = new Expression(o, "get", new Object[] { o3 });
                Expression o5 = new Expression(o2, "get", new Object[] { o3 });
                try {
                    Object value = o4.getValue();
                    Object value2 = o5.getValue();
                    encoder.writeExpression(o4);
                    if (!Objects.equals(value2, encoder.get(value))) {
                        DefaultPersistenceDelegate.invokeStatement(o, "set", new Object[] { o3, value }, encoder);
                    }
                }
                catch (Exception ex) {
                    encoder.getExceptionListener().exceptionThrown(ex);
                }
            }
            for (; i < size; ++i) {
                DefaultPersistenceDelegate.invokeStatement(o, "add", new Object[] { list.get(i) }, encoder);
            }
        }
    }
    
    static class java_util_Map_PersistenceDelegate extends DefaultPersistenceDelegate
    {
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
            Map<Object, Object> map = (Map)o;
            Map<Object, Object> map2 = (Map)o2;
            if (map2 != null) {
                for (Object o3 : map2.keySet().toArray()) {
                    if (!map.containsKey(o3)) {
                        DefaultPersistenceDelegate.invokeStatement(o, "remove", new Object[] { o3 }, encoder);
                    }
                }
            }
            for (Object next : map.keySet()) {
                Expression o4 = new Expression(o, "get", new Object[] { next });
                Expression o5 = new Expression(o2, "get", new Object[] { next });
                try {
                    Object value = o4.getValue();
                    Object value2 = o5.getValue();
                    encoder.writeExpression(o4);
                    if (!Objects.equals(value2, encoder.get(value))) {
                        DefaultPersistenceDelegate.invokeStatement(o, "put", new Object[] { next, value }, encoder);
                    }
                    else if (value2 == null && !map2.containsKey(next)) {
                        DefaultPersistenceDelegate.invokeStatement(o, "put", new Object[] { next, value }, encoder);
                    }
                    else {
                        continue;
                    }
                }
                catch (Exception ex) {
                    encoder.getExceptionListener().exceptionThrown(ex);
                }
            }
        }
    }
    
    static final class java_util_AbstractCollection_PersistenceDelegate extends MetaData.java_util_Collection_PersistenceDelegate
    {
    }
    
    static final class java_util_AbstractList_PersistenceDelegate extends MetaData.java_util_List_PersistenceDelegate
    {
    }
    
    static final class java_util_AbstractMap_PersistenceDelegate extends MetaData.java_util_Map_PersistenceDelegate
    {
    }
    
    static final class java_util_Hashtable_PersistenceDelegate extends MetaData.java_util_Map_PersistenceDelegate
    {
    }
    
    static final class java_beans_beancontext_BeanContextSupport_PersistenceDelegate extends MetaData.java_util_Collection_PersistenceDelegate
    {
    }
    
    static final class java_awt_Insets_PersistenceDelegate extends PersistenceDelegate
    {
        @Override
        protected boolean mutatesTo(Object o, Object obj) {
            return o.equals(obj);
        }
        
        @Override
        protected Expression instantiate(Object o, Encoder encoder) {
            Insets insets = (Insets)o;
            return new Expression(insets, insets.getClass(), "new", new Object[] { Integer.valueOf(insets.top), Integer.valueOf(insets.left), Integer.valueOf(insets.bottom), Integer.valueOf(insets.right) });
        }
    }
    
    static final class java_awt_Font_PersistenceDelegate extends PersistenceDelegate
    {
        @Override
        protected boolean mutatesTo(Object o, Object obj) {
            return o.equals(obj);
        }
        
        @Override
        protected Expression instantiate(Object o, Encoder encoder) {
            Font font = (Font)o;
            int n = 0;
            String s = null;
            int i = 0;
            int intValue = 12;
            Map<TextAttribute, Object> attributes = font.getAttributes();
            HashMap attributes2 = new HashMap(attributes.size());
            for (TextAttribute textAttribute : attributes.keySet()) {
                Object value = attributes.get(textAttribute);
                if (value != null) {
                    attributes2.put(textAttribute, value);
                }
                if (textAttribute == TextAttribute.FAMILY) {
                    if ((value instanceof String)) {
                        ++n;
                        s = (String)value;
                    }
                    else {
                        continue;
                    }
                }
                else if (textAttribute == TextAttribute.WEIGHT) {
                    if (TextAttribute.WEIGHT_REGULAR.equals(value)) {
                        ++n;
                    }
                    else if (TextAttribute.WEIGHT_BOLD.equals(value)) {
                        ++n;
                        i |= 0x1;
                    }
                    else {
                        continue;
                    }
                }
                else if (textAttribute == TextAttribute.POSTURE) {
                    if (TextAttribute.POSTURE_REGULAR.equals(value)) {
                        ++n;
                    }
                    else if (TextAttribute.POSTURE_OBLIQUE.equals(value)) {
                        ++n;
                        i |= 0x2;
                    }
                    else {
                        continue;
                    }
                }
                else if (textAttribute == TextAttribute.SIZE && value instanceof Number) {
                    Number n2 = (Number)value;
                    intValue = n2.intValue();
                    if ((float)intValue == n2.floatValue()) {
                        ++n;
                    }
                    else {
                        continue;
                    }
                }
                else {
                    continue;
                }
            }
            Class<Object> class1 = font.getClass();
            if (n == attributes2.size()) {
                return new Expression(font, class1, "new", new Object[] { s, Integer.valueOf(i), Integer.valueOf(intValue) });
            }
            if (class1 == Font.class) {
                return new Expression(font, class1, "getFont", new Object[] { attributes2 });
            }
            return new Expression(font, class1, "new", new Object[] { Font.getFont(attributes2) });
        }
    }
    
    static final class java_awt_AWTKeyStroke_PersistenceDelegate extends PersistenceDelegate
    {
        @Override
        protected boolean mutatesTo(Object o, Object obj) {
            return o.equals(obj);
        }
        
        @Override
        protected Expression instantiate(Object o, Encoder encoder) {
            AWTKeyStroke obj = (AWTKeyStroke)o;
            int keyChar = obj.getKeyChar();
            int keyCode = obj.getKeyCode();
            int modifiers = obj.getModifiers();
            int onKeyRelease = obj.isOnKeyRelease();
            Object[] array = null;
            if (keyChar == '\uffff') {
                array = (onKeyRelease ? new Object[] { Integer.valueOf(keyCode), Integer.valueOf(modifiers), Boolean.valueOf((boolean)onKeyRelease) } : new Object[] { Integer.valueOf(keyCode), Integer.valueOf(modifiers) });
            }
            else if (keyCode == 0) {
                if (onKeyRelease == 0) {
                    array = ((modifiers == 0) ? new Object[] { Character.valueOf((char)keyChar) } : new Object[] { Character.valueOf((char)keyChar), Integer.valueOf(modifiers) });
                }
                else if (modifiers == 0) {
                    array = new Object[] { Character.valueOf((char)keyChar), Boolean.valueOf((boolean)onKeyRelease) };
                }
            }
            if (array == null) {
                throw new IllegalStateException("Unsupported KeyStroke: " + obj);
            }
            Class<Object> class1 = obj.getClass();
            String str = class1.getName();
            int beginIndex = str.lastIndexOf(46) + 1;
            if (beginIndex > 0) {
                str = str.substring(beginIndex);
            }
            return new Expression(obj, class1, "get" + str, array);
        }
    }
    
    static class StaticFieldsPersistenceDelegate extends PersistenceDelegate
    {
        protected void installFields(Encoder encoder, Class<?> clazz) {
            if (Modifier.isPublic(clazz.getModifiers()) && ReflectUtil.isPackageAccessible(clazz)) {
                Field[] fields = clazz.getFields();
                for (int i = 0; i < fields.length; ++i) {
                    Field field = fields[i];
                    if (Object.class.isAssignableFrom(field.getType())) {
                        encoder.writeExpression(new Expression(field, "get", new Object[] { null }));
                    }
                }
            }
        }
        
        @Override
        protected Expression instantiate(Object obj, Encoder encoder) {
            throw new RuntimeException("Unrecognized instance: " + obj);
        }
        
        @Override
        public void writeObject(Object o, Encoder encoder) {
            if (encoder.getAttribute(this) == null) {
                encoder.setAttribute(this, Boolean.TRUE);
                this.installFields(encoder, o.getClass());
            }
            super.writeObject(o, encoder);
        }
    }
    
    static final class java_awt_SystemColor_PersistenceDelegate extends MetaData.StaticFieldsPersistenceDelegate
    {
    }
    
    static final class java_awt_font_TextAttribute_PersistenceDelegate extends MetaData.StaticFieldsPersistenceDelegate
    {
    }
    
    static final class java_awt_MenuShortcut_PersistenceDelegate extends PersistenceDelegate
    {
        @Override
        protected boolean mutatesTo(Object o, Object obj) {
            return o.equals(obj);
        }
        
        @Override
        protected Expression instantiate(Object o, Encoder encoder) {
            MenuShortcut menuShortcut = (MenuShortcut)o;
            return new Expression(o, menuShortcut.getClass(), "new", new Object[] { new Integer(menuShortcut.getKey()), Boolean.valueOf(menuShortcut.usesShiftModifier()) });
        }
    }
    
    static final class java_awt_Component_PersistenceDelegate extends DefaultPersistenceDelegate
    {
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
            super.initialize(clazz, o, o2, encoder);
            Component component = (Component)o;
            Component component2 = (Component)o2;
            if (!(o instanceof Window)) {
                Color a = component.isBackgroundSet() ? component.getBackground() : null;
                if (!Objects.equals(a, component2.isBackgroundSet() ? component2.getBackground() : null)) {
                    DefaultPersistenceDelegate.invokeStatement(o, "setBackground", new Object[] { a }, encoder);
                }
                Color a2 = component.isForegroundSet() ? component.getForeground() : null;
                if (!Objects.equals(a2, component2.isForegroundSet() ? component2.getForeground() : null)) {
                    DefaultPersistenceDelegate.invokeStatement(o, "setForeground", new Object[] { a2 }, encoder);
                }
                Font a3 = component.isFontSet() ? component.getFont() : null;
                if (!Objects.equals(a3, component2.isFontSet() ? component2.getFont() : null)) {
                    DefaultPersistenceDelegate.invokeStatement(o, "setFont", new Object[] { a3 }, encoder);
                }
            }
            Container parent = component.getParent();
            if (parent == null || parent.getLayout() == null) {
                int equals = component.getLocation().equals(component2.getLocation());
                int equals2 = component.getSize().equals(component2.getSize());
                if (equals == 0 && equals2 == 0) {
                    DefaultPersistenceDelegate.invokeStatement(o, "setBounds", new Object[] { component.getBounds() }, encoder);
                }
                else if (equals == 0) {
                    DefaultPersistenceDelegate.invokeStatement(o, "setLocation", new Object[] { component.getLocation() }, encoder);
                }
                else if (equals2 == 0) {
                    DefaultPersistenceDelegate.invokeStatement(o, "setSize", new Object[] { component.getSize() }, encoder);
                }
            }
        }
    }
    
    static final class java_awt_Container_PersistenceDelegate extends DefaultPersistenceDelegate
    {
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
            super.initialize(clazz, o, o2, encoder);
            if ((o instanceof JScrollPane)) {
                return;
            }
            Container container = (Container)o;
            Component[] components = container.getComponents();
            Container container2 = (Container)o2;
            Component[] array = (container2 == null) ? new Component[0] : container2.getComponents();
            BorderLayout borderLayout = (container.getLayout() instanceof BorderLayout) ? ((BorderLayout)container.getLayout()) : null;
            JLayeredPane layeredPane = (o instanceof JLayeredPane) ? ((JLayeredPane)o) : null;
            for (int i = array.length; i < components.length; ++i) {
                DefaultPersistenceDelegate.invokeStatement(o, "add", (borderLayout != null) ? new Object[] { components[i], borderLayout.getConstraints(components[i]) } : ((layeredPane != null) ? new Object[] { components[i], Integer.valueOf(layeredPane.getLayer(components[i])), Integer.valueOf(-1) } : new Object[] { components[i] }), encoder);
            }
            return;
        }
    }
    
    static final class java_awt_Choice_PersistenceDelegate extends DefaultPersistenceDelegate
    {
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
            super.initialize(clazz, o, o2, encoder);
            Choice choice = (Choice)o;
            for (int i = ((Choice)o2).getItemCount(); i < choice.getItemCount(); ++i) {
                DefaultPersistenceDelegate.invokeStatement(o, "add", new Object[] { choice.getItem(i) }, encoder);
            }
        }
    }
    
    static final class java_awt_Menu_PersistenceDelegate extends DefaultPersistenceDelegate
    {
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
            super.initialize(clazz, o, o2, encoder);
            Menu menu = (Menu)o;
            for (int i = ((Menu)o2).getItemCount(); i < menu.getItemCount(); ++i) {
                DefaultPersistenceDelegate.invokeStatement(o, "add", new Object[] { menu.getItem(i) }, encoder);
            }
        }
    }
    
    static final class java_awt_MenuBar_PersistenceDelegate extends DefaultPersistenceDelegate
    {
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
            super.initialize(clazz, o, o2, encoder);
            MenuBar menuBar = (MenuBar)o;
            for (int i = ((MenuBar)o2).getMenuCount(); i < menuBar.getMenuCount(); ++i) {
                DefaultPersistenceDelegate.invokeStatement(o, "add", new Object[] { menuBar.getMenu(i) }, encoder);
            }
        }
    }
    
    static final class java_awt_List_PersistenceDelegate extends DefaultPersistenceDelegate
    {
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
            super.initialize(clazz, o, o2, encoder);
            java.awt.List list = (java.awt.List)o;
            for (int i = ((java.awt.List)o2).getItemCount(); i < list.getItemCount(); ++i) {
                DefaultPersistenceDelegate.invokeStatement(o, "add", new Object[] { list.getItem(i) }, encoder);
            }
        }
    }
    
    static final class java_awt_BorderLayout_PersistenceDelegate extends DefaultPersistenceDelegate
    {
        private static final String[] CONSTRAINTS;
        
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
            super.initialize(clazz, o, o2, encoder);
            BorderLayout borderLayout = (BorderLayout)o;
            BorderLayout borderLayout2 = (BorderLayout)o2;
            for (String s : MetaData.java_awt_BorderLayout_PersistenceDelegate.CONSTRAINTS) {
                Component layoutComponent = borderLayout.getLayoutComponent(s);
                Component layoutComponent2 = borderLayout2.getLayoutComponent(s);
                if (layoutComponent != null && layoutComponent2 == null) {
                    DefaultPersistenceDelegate.invokeStatement(o, "addLayoutComponent", new Object[] { layoutComponent, s }, encoder);
                }
            }
        }
        
        static {
            CONSTRAINTS = new String[] { "North", "South", "East", "West", "Center", "First", "Last", "Before", "After" };
        }
    }
    
    static final class java_awt_CardLayout_PersistenceDelegate extends DefaultPersistenceDelegate
    {
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
            super.initialize(clazz, o, o2, encoder);
            if (getVector(o2).isEmpty()) {
                for (Object next : getVector(o)) {
                    DefaultPersistenceDelegate.invokeStatement(o, "addLayoutComponent", new Object[] { MetaData.getPrivateFieldValue(next, "java.awt.CardLayout$Card.name"), MetaData.getPrivateFieldValue(next, "java.awt.CardLayout$Card.comp") }, encoder);
                }
            }
        }
        
        @Override
        protected boolean mutatesTo(Object o, Object o2) {
            return super.mutatesTo(o, o2) && getVector(o2).isEmpty();
        }
        
        private static Vector<?> getVector(Object o) {
            return (Vector)MetaData.getPrivateFieldValue(o, "java.awt.CardLayout.vector");
        }
    }
    
    static final class java_awt_GridBagLayout_PersistenceDelegate extends DefaultPersistenceDelegate
    {
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
            super.initialize(clazz, o, o2, encoder);
            if (getHashtable(o2).isEmpty()) {
                for (Map.Entry entry : getHashtable(o).entrySet()) {
                    DefaultPersistenceDelegate.invokeStatement(o, "addLayoutComponent", new Object[] { entry.getKey(), entry.getValue() }, encoder);
                }
            }
        }
        
        @Override
        protected boolean mutatesTo(Object o, Object o2) {
            return super.mutatesTo(o, o2) && getHashtable(o2).isEmpty();
        }
        
        private static Hashtable<?, ?> getHashtable(Object o) {
            return (Hashtable)MetaData.getPrivateFieldValue(o, "java.awt.GridBagLayout.comptable");
        }
    }
    
    static final class javax_swing_JFrame_PersistenceDelegate extends DefaultPersistenceDelegate
    {
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
            super.initialize(clazz, o, o2, encoder);
            Window window = (Window)o;
            Window window2 = (Window)o2;
            int visible = window.isVisible();
            if ((window2.isVisible() ? 1 : 0) != visible) {
                int executeStatements = encoder.executeStatements;
                encoder.executeStatements = false;
                DefaultPersistenceDelegate.invokeStatement(o, "setVisible", new Object[] { Boolean.valueOf((boolean)visible) }, encoder);
                encoder.executeStatements = executeStatements;
            }
        }
    }
    
    static final class javax_swing_DefaultListModel_PersistenceDelegate extends DefaultPersistenceDelegate
    {
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
            super.initialize(clazz, o, o2, encoder);
            DefaultListModel<Object> defaultListModel = (DefaultListModel)o;
            for (int i = ((DefaultListModel)o2).getSize(); i < defaultListModel.getSize(); ++i) {
                DefaultPersistenceDelegate.invokeStatement(o, "add", new Object[] { defaultListModel.getElementAt(i) }, encoder);
            }
        }
    }
    
    static final class javax_swing_DefaultComboBoxModel_PersistenceDelegate extends DefaultPersistenceDelegate
    {
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
            super.initialize(clazz, o, o2, encoder);
            DefaultComboBoxModel<Object> defaultComboBoxModel = (DefaultComboBoxModel)o;
            for (int i = 0; i < defaultComboBoxModel.getSize(); ++i) {
                DefaultPersistenceDelegate.invokeStatement(o, "addElement", new Object[] { defaultComboBoxModel.getElementAt(i) }, encoder);
            }
        }
    }
    
    static final class javax_swing_tree_DefaultMutableTreeNode_PersistenceDelegate extends DefaultPersistenceDelegate
    {
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
            super.initialize(clazz, o, o2, encoder);
            DefaultMutableTreeNode defaultMutableTreeNode = (DefaultMutableTreeNode)o;
            for (int i = ((DefaultMutableTreeNode)o2).getChildCount(); i < defaultMutableTreeNode.getChildCount(); ++i) {
                DefaultPersistenceDelegate.invokeStatement(o, "add", new Object[] { defaultMutableTreeNode.getChildAt(i) }, encoder);
            }
        }
    }
    
    static final class javax_swing_ToolTipManager_PersistenceDelegate extends PersistenceDelegate
    {
        @Override
        protected Expression instantiate(Object o, Encoder encoder) {
            return new Expression(o, ToolTipManager.class, "sharedInstance", new Object[0]);
        }
    }
    
    static final class javax_swing_JTabbedPane_PersistenceDelegate extends DefaultPersistenceDelegate
    {
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
            super.initialize(clazz, o, o2, encoder);
            JTabbedPane tabbedPane = (JTabbedPane)o;
            for (int i = 0; i < tabbedPane.getTabCount(); ++i) {
                DefaultPersistenceDelegate.invokeStatement(o, "addTab", new Object[] { tabbedPane.getTitleAt(i), tabbedPane.getIconAt(i), tabbedPane.getComponentAt(i) }, encoder);
            }
        }
    }
    
    static final class javax_swing_Box_PersistenceDelegate extends DefaultPersistenceDelegate
    {
        @Override
        protected boolean mutatesTo(Object o, Object o2) {
            return super.mutatesTo(o, o2) && this.getAxis(o).equals(this.getAxis(o2));
        }
        
        @Override
        protected Expression instantiate(Object o, Encoder encoder) {
            return new Expression(o, o.getClass(), "new", new Object[] { this.getAxis(o) });
        }
        
        private Integer getAxis(Object o) {
            return (Integer)MetaData.getPrivateFieldValue(((Box)o).getLayout(), "javax.swing.BoxLayout.axis");
        }
    }
    
    static final class javax_swing_JMenu_PersistenceDelegate extends DefaultPersistenceDelegate
    {
        @Override
        protected void initialize(Class<?> clazz, Object o, Object o2, Encoder encoder) {
            super.initialize(clazz, o, o2, encoder);
            Component[] menuComponents = ((JMenu)o).getMenuComponents();
            for (int i = 0; i < menuComponents.length; ++i) {
                DefaultPersistenceDelegate.invokeStatement(o, "add", new Object[] { menuComponents[i] }, encoder);
            }
        }
    }
    
    static final class javax_swing_border_MatteBorder_PersistenceDelegate extends PersistenceDelegate
    {
        @Override
        protected Expression instantiate(Object o, Encoder encoder) {
            MatteBorder matteBorder = (MatteBorder)o;
            Insets borderInsets = matteBorder.getBorderInsets();
            Icon icon = matteBorder.getTileIcon();
            if (icon == null) {
                icon = matteBorder.getMatteColor();
            }
            return new Expression(matteBorder, matteBorder.getClass(), "new", new Object[] { Integer.valueOf(borderInsets.top), Integer.valueOf(borderInsets.left), Integer.valueOf(borderInsets.bottom), Integer.valueOf(borderInsets.right), icon });
        }
    }
    
    static final class sun_swing_PrintColorUIResource_PersistenceDelegate extends PersistenceDelegate
    {
        @Override
        protected boolean mutatesTo(Object o, Object obj) {
            return o.equals(obj);
        }
        
        @Override
        protected Expression instantiate(Object o, Encoder encoder) {
            Color color = (Color)o;
            return new Expression(color, ColorUIResource.class, "new", new Object[] { Integer.valueOf(color.getRGB()) });
        }
    }
}
